@@ -1,6 +1,5 @@
 package Hash::AutoHash;
-our $VERSION='1.17_01';
-$VERSION=eval $VERSION;		# I think this is the accepted idiom..
+our $VERSION='1.01';
 
 #################################################################################
 #
@@ -63,50 +62,47 @@ sub new {
   my $helper_class=$class_or_self.'::helper';
   $helper_class->_new($class_or_self,@_);
 }
-# NG 12-09-02: no longer possible to use method notation for keys with same names as methods
-#              inherited from UNIVERSAL. 'Cuz as of Perl 5.9.3, calling UNIVERSAL methods as
-#              functions is deprecated and developers encouraged to use method form instead.
-# sub can {
-#   my $class_or_self=shift;
-#   if (ref $class_or_self) { 
-#     # called as object method. access hash slot via AUTOLOAD
-#     $AUTOLOAD='can';
-#     return $class_or_self->AUTOLOAD(@_);
-#   }
-#   # called as class method. do regular 'can' via base class
-#   return $class_or_self->SUPER::can(@_);
-# }
-# sub isa {
-#   my $class_or_self=shift;
-#   if (ref $class_or_self) { 
-#     # called as object method. access hash slot via AUTOLOAD
-#     $AUTOLOAD='isa';
-#     return $class_or_self->AUTOLOAD(@_);
-#   }
-#   # called as function or class method. do regular 'isa' via base class
-#   return $class_or_self->SUPER::isa(@_);
-# }
-# sub DOES {			# in perl 5.10, UNIVERSAL provides this
-#   my $class_or_self=shift;
-#   if (ref $class_or_self) { 
-#     # called as object method. access hash slot via AUTOLOAD
-#     $AUTOLOAD='DOES';
-#     return $class_or_self->AUTOLOAD(@_);
-#   }
-#   # called as function or class method. do regular 'DOES' via base class
-#   # illegal and will die in perls < 5.10 
-#   return $class_or_self->SUPER::DOES(@_);
-# }
-# sub VERSION {
-#   my $class_or_self=shift;
-#   if (ref $class_or_self) { 
-#     # called as object method. access hash slot via AUTOLOAD
-#     $AUTOLOAD='VERSION';
-#     return $class_or_self->AUTOLOAD(@_);
-#   }
-#   # called as function or class method. do regular 'VERSION' via base class
-#   return $class_or_self->SUPER::VERSION(@_);
-# }
+sub can {
+  my $class_or_self=shift;
+  if (ref $class_or_self) { 
+    # called as object method. access hash slot via AUTOLOAD
+    $AUTOLOAD='can';
+    return $class_or_self->AUTOLOAD(@_);
+  }
+  # called as class method. do regular 'can' via base class
+  return $class_or_self->SUPER::can(@_);
+}
+sub isa {
+  my $class_or_self=shift;
+  if (ref $class_or_self) { 
+    # called as object method. access hash slot via AUTOLOAD
+    $AUTOLOAD='isa';
+    return $class_or_self->AUTOLOAD(@_);
+  }
+  # called as function or class method. do regular 'isa' via base class
+  return $class_or_self->SUPER::isa(@_);
+}
+sub DOES {			# in perl 5.10, UNIVERSAL provides this
+  my $class_or_self=shift;
+  if (ref $class_or_self) { 
+    # called as object method. access hash slot via AUTOLOAD
+    $AUTOLOAD='DOES';
+    return $class_or_self->AUTOLOAD(@_);
+  }
+  # called as function or class method. do regular 'DOES' via base class
+  # illegal and will die in perls < 5.10 
+  return $class_or_self->SUPER::DOES(@_);
+}
+sub VERSION {
+  my $class_or_self=shift;
+  if (ref $class_or_self) { 
+    # called as object method. access hash slot via AUTOLOAD
+    $AUTOLOAD='VERSION';
+    return $class_or_self->AUTOLOAD(@_);
+  }
+  # called as function or class method. do regular 'VERSION' via base class
+  return $class_or_self->SUPER::VERSION(@_);
+}
 sub DESTROY  { 
   # CAUTION: do NOT shift - need $_[0] intact
   if (ref($_[0])) {
@@ -152,7 +148,6 @@ sub AUTOLOAD {
 # functions herein (except _new, _autoload) are exportable by Hash::AutoHash
 #################################################################################
 package Hash::AutoHash::helper;
-our $VERSION=$Hash::AutoHash::VERSION;
 use strict;
 use Carp;
 use Scalar::Util qw(blessed readonly reftype);
@@ -245,9 +240,7 @@ sub EXPORT_OK {
   {
     no strict 'refs';
     $export_ok_list=\@{$class.'::EXPORT_OK'};
-    # NG 12-11-29: 'defined @array' deprecated in 5.16 or so
-    # return @$export_ok_list if defined @$export_ok_list;
-    return @$export_ok_list if @$export_ok_list;
+    return @$export_ok_list if defined @$export_ok_list;
     $export_ok_hash=\%{$class.'::EXPORT_OK'}; 
     @isa=@{$helper_class.'::ISA'};
     @normal_export_ok=@{$class.'::NORMAL_EXPORT_OK'};
@@ -294,9 +287,7 @@ sub SUBCLASS_EXPORT_OK {
   } 
   # for subclasses, @SUBCLASS_EXPORT_OK must be computed
   my $subclass_export_ok=\@{$class.'::SUBCLASS_EXPORT_OK'};
-  # NG 12-11-29: 'defined @array' deprecated in 5.16 or so
-  # return @$subclass_export_ok if defined @$subclass_export_ok;
-  return @$subclass_export_ok if @$subclass_export_ok;
+  return @$subclass_export_ok if defined @$subclass_export_ok;
   return @$subclass_export_ok=$helper_class->EXPORT_OK;
 }
 
@@ -571,7 +562,6 @@ sub autohash_untie {
 #   none could connect new variable to old one without changing the type of old
 #################################################################################
 package Hash::AutoHash::alias;
-our $VERSION=$Hash::AutoHash::VERSION;
 use strict;
 use Tie::Hash;
 our @ISA=qw(Tie::ExtraHash);
@@ -590,7 +580,7 @@ Hash::AutoHash - Object-oriented access to real and tied hashes
 
 =head1 VERSION
 
-Version 1.17
+Version 1.00
 
 =head1 SYNOPSIS
 
@@ -657,7 +647,8 @@ allows a modular division of labor: this class is generic and treats
 all hashes the same; any special semantics come from the tied hash.
 
 The class has a 'new' method but also supplies several functions for
-constructing new Hash::AutoHash objects.
+constructing new Hash::AutoHash objects.  Except in the simplest
+cases, we recommend using the constructor functions rather than 'new'.
 
 The constructor functions shown in the SYNOPSIS are all you need for
 typical uses.  autohash_hash creates a new 'real' (ie, not tied)
@@ -682,25 +673,9 @@ And you can add new elements using either notation:
   $autohash->{first_name}='Joe';
   $autohash->last_name('Plumber');
 
-CAUTIONS
-
-=over 2
-
-=item * When using method notation, keys must be
+One caution is that when using method notation, keys must be
 syntactically legal method names and cannot include 'funny' characters.
 
-=item * INCOMPATIBLE CHANGE: As of version 1.14, it is no longer
-possible to use method notation for keys with the same names
-as methods inherited from UNIVERSAL (the base class of
-everything). These are 'can', 'isa', 'DOES', and 'VERSION'.
-The reason is that as of Perl 5.9.3, calling UNIVERSAL
-methods as functions is deprecated and developers are
-encouraged to use method form instead. Previous versions of
-AutoHash are incompatible with CPAN modules that adopt this
-style.
-
-=back
-
 Nested structures work straightforwardly. If a value is a
 Hash::AutoHash object, you can use a series of '->' operators
 to get to its elements.
@@ -797,26 +772,49 @@ object is not tied.
 Hash::AutoHash provides all of its capabilities through class
 methods (these are methods, such as 'new', that are invoked on the
 class rather than on individual objects) or through functions that
-must be imported into the caller's namespace.  In most cases, a method invoked on
+must be imported into the caller's namespace.  Any method invoked on
 an object is interpreted as a request to access or change an element
-of the underlying hash. 
+of the underlying hash.
 
-CAUTION: As of version 1.14, it is not possible to use method
-notation for keys with the same names as methods inherited from
-UNIVERSAL (the base class of everything). These are 'can', 'isa',
-'DOES', and 'VERSION'.  The reason is that as of Perl 5.9.3, calling
-UNIVERSAL methods as functions is deprecated and developers are
-encouraged to use method form instead. Previous versions of AutoHash
-are incompatible with CPAN modules that adopt this style.
+Special care is needed with methods inherited from UNIVERSAL (the base
+class of everything) and methods used implicitly by Perl to implement
+common features ('import', 'AUTOLOAD', 'DESTROY').
 
-Special care is needed with methods used implicitly by Perl to
-implement common features ('import', 'AUTOLOAD', 'DESTROY'). 
+UNIVERSAL defines four methods in perl 5.10 -- 'can', 'isa', 'DOES',
+and 'VERSION' -- and three is earlier versions -- 'can', 'isa', and
+'VERSION'.  These are logically class methods, since they give the
+same result for every object in a given class, but the built-in
+implementation allows them to be invoked on objects. We have chosen to
+override the standard implementation: if the method is invoked on a
+class, we pass it to UNIVERSAL, but if it is invoked on an object, we
+interpret it as a request to access or change an element of the hash.
 
-'import' is usually invoked by Perl as a class method when processing
-'use' statements to import functions into the caller's namespace. We
-preserve this behavior but when invoked on an object, we interpret the
-call as a request to access or change the element of the underlying
-hash whose kye is 'import'.
+For example, the following two statements
+
+  can Hash::AutoHash('import')
+  Hash::AutoHash->can('import')
+
+invoke the 'can' method on the class Hash::AutoHash; the code
+notices that the target is a class and forwards the call to the 'can'
+method in UNIVERSAL which tells whether the module has an 'import'
+method (it does).  On the other hand,
+
+  $autohash->can('import')
+
+invokes the 'can' method on the $autohash object (assuming, of course,
+that's what $autohash contains); the code notices that the target is a
+Hash::AutoHash object and sets the value of the hash element 'can'
+to 'import'. 
+
+CAUTION: This will break code that expects the UNIVERSAL methods
+to work in the standard way.  If this proves to be a problem, we
+will provide a way to revert the offending behavior.
+
+Our implementation of 'import' is similar. This method is invoked by
+Perl as a class method when processing 'use' statements. When called
+in this manner, it imports functions into the caller's namespace as
+expected, but when invoked on an object, it accesses the underlying
+hash.
 
 'AUTOLOAD' and 'DESTROY' pose different problems, since they are
 logically object methods.  Fortunately, Perl leaves enough clues to
@@ -840,16 +838,18 @@ the code accesses the hash element named 'new'.
 
 =head2 Constructors
 
-Hash::AutoHash provides a number of constructor functions as well as a
-'new' method which is simply a front-end for the constructor
-functions. To use the constructor functions, you must import them into
-the caller's namespace using the common Perl idiom of listing the
-desired functions in a 'use' statement.
+We recommend the use of constructor functions rather than 'new' which
+is simply a front-end for the constructor functions. To use these
+functions, you must import them into the caller's namespace using the
+common Perl idiom of listing the desired functions in a 'use'
+statement.
 
- use Hash::AutoHash qw(autohash_hash autohash_tie autohash_wrap autohash_wrapobj
-                       autohash_wraptie autohash_new);
+=head3 Typical usage
 
-=head3 autohash_hash
+In typical cases, the two constructor functions described in this
+section are all that are needed.
+
+ use Hash::AutoHash qw(autohash_hash autohash_tie);
 
  Title   : autohash_hash
  Usage   : $autohash=autohash_hash name=>'Joe',hobbies=>['hiking','cooking']
@@ -858,8 +858,6 @@ desired functions in a 'use' statement.
  Returns : Hash::AutoHash object
  Args    : Optional list of key=>value pairs
 
-=head3 autohash_tie
-
  Title   : autohash_tie
  Usage   : $autohash=autohash_tie Tie::Hash::MultiValue
  Function: Create a tied Hash::AutoHash object
@@ -900,7 +898,7 @@ tied object, a process we call wrapping. Once the linkage is made, the
 contents of the object and hash will be identical; any changes made to
 one will be reflected in the other.
 
-=head4 autohash_wrap
+ use Hash::AutoHash qw(autohash_wrap autohash_wrapobj autohash_wraptie);
 
  Title   : autohash_wrap
  Usage   : $autohash=autohash_wrap %hash,name=>'Joe',hobbies=>['hiking','cooking']
@@ -917,8 +915,6 @@ one will be reflected in the other.
            constructed object will be tied to an object of type 
            Hash::AutoHash::dup which implements the linking.
 
-=head4 autohash_wrapobj
-
  Title   : autohash_wrapobj
  Usage   : $autohash=autohash_wrapobj $tied_object,name=>'Joe',hobbies=>'hiking'
  Function: Create a Hash::AutoHash object linked to a tied hash given 
@@ -931,20 +927,16 @@ one will be reflected in the other.
  Args    : Object implementing a tied hash and optional list of key=>value 
            pairs. The key=>value pairs set further elements of the object (and 
            hash).
+ Notes   : Here is another, perhaps more typical, illustration of this function.
+           $autohash=autohash_wrapobj tie %hash,'Tie::Hash::MultiValue'
 
-Here is another, perhaps more typical, illustration of autohash_wrapobj.
-
-  $autohash=autohash_wrapobj tie %hash,'Tie::Hash::MultiValue'
-
-You can set initial values in the constructed object by including them
-as parameters to the function, using parentheses to keep them separate
-from the parameters to 'tie'.  All the parentheses in the example
-below are necessary.
-
-  $autohash=autohash_wrapobj ((tie %hash,'Tie::Hash::MultiValue'),
-                              name=>'Joe',hobbies=>'hiking',hobbies=>'cooking')
-
-=head4 autohash_wraptie
+           You can set initial values in the constructed object by including
+           them as parameters to the function, using parentheses to keep them 
+           separate from the parameters to 'tie'.  All the parentheses in the 
+           example below are necessary.
+           $autohash=autohash_wrapobj ((tie %hash,'Tie::Hash::MultiValue'),
+                                       name=>'Joe',
+                                       hobbies=>'hiking',hobbies=>'cooking')
 
  Title   : autohash_wraptie
  Usage   : $autohash=autohash_wraptie %hash,Tie::Hash::MultiValue
@@ -957,11 +949,14 @@ below are necessary.
  Notes   : This is equivalent to
            $autohash=autohash_wrapobj tie %hash,'Tie::Hash::MultiValue'
 
-=head3 new
+=head2 'new' method and autohash_new function
 
-'new' and L<autohash_new> are front-ends to the other constructor
-functions. To accommodate the diversity of the other functions, the
-parameter syntax makes some assumptions and is not completely general.
+The class provides a 'new' method mainly because programmers expect
+it, and an equivalent autohash_new function for stylistic
+consistency. 'new' and autohash_new are front-ends to the other
+constructor functions. To accommodate the diversity of the other
+functions, the parameter syntax makes some assumptions and is not
+completely general.
 
  Title   : new 
  Usage   : $autohash=new Hash::AutoHash 
@@ -1002,12 +997,7 @@ parameter syntax makes some assumptions and is not completely general.
              class implementing the tied hash (quotes are required) followed by
              additional parameters for the the TIEHASH constructor.
 
-=head3 autohash_new
-
-Like L<new>, autohash_new is a front-end to the other constructor
-functions. We provide it for stylistic consistency. To accommodate the
-diversity of the other functions, the parameter syntax makes some
-assumptions and is not completely general.
+ use Hash::AutoHash qw(autohash_new);
 
  Title   : autohash_new 
  Usage   : $autohash=autohash_new name=>'Joe',hobbies=>['hiking','cooking']
@@ -1027,7 +1017,7 @@ Function: same as 'new'
  Returns : Hash::AutoHash object
  Args    : same as 'new'
 
-=head2 Aliasing: autohash_alias
+=head2 Aliasing
 
 You can alias a Hash::AutoHash object to a regular hash to
 avoid the need to dereference the variable when using hash
@@ -1084,13 +1074,9 @@ repeatedly.
   my @keys=keys %hash;          # instead of keys %$autohash
 
 The class also provides two functions for wholesale manipulation of
-arguments.  To use these functions, you must import them into the
-caller's namespace using the common Perl idiom of listing the desired
-functions in a 'use' statement.
+arguments.
 
  use Hash::AutoHash qw(autohash_get autohash_set);
-
-=head3 autohash_get
  
  Title   : autohash_get
  Usage   : ($name,$hobbies)=autohash_get($autohash,qw(name hobbies))
@@ -1098,8 +1084,6 @@ functions in a 'use' statement.
  Args    : Hash::AutoHash object and list of keys
  Returns : list of argument values
 
-=head3 autohash_set
-
  Title   : autohash_set
  Usage   : autohash_set($autohash,name=>'Joe Plumber',first_name=>'Joe')
            -- OR --
@@ -1113,41 +1097,31 @@ functions in a 'use' statement.
 
 These functions provide hash-like operations on Hash::AutoHash
 objects. These are useful if you want to avoid hash notation all
-together. To use these functions, you must import them into the
-caller's namespace using the common Perl idiom of listing the desired
-functions in a 'use' statement.
+together.
 
  use Hash::AutoHash 
     qw(autohash_clear autohash_delete autohash_each autohash_exists 
        autohash_keys autohash_values 
        autohash_count autohash_empty autohash_notempty);
 
-=head3 autohash_clear
-
  Title   : autohash_clear
  Usage   : autohash_clear($autohash)
  Function: Delete entire contents of $autohash
  Args    : Hash::AutoHash object
  Returns : nothing
 
-=head3 autohash_delete
-
  Title   : autohash_delete
  Usage   : autohash_delete($autohash,@keys)
  Function: Delete keys and their values from $autohash.
  Args    : Hash::AutoHash object, list of keys
  Returns : nothing
 
-=head3 autohash_exists
-
  Title   : autohash_exists
  Usage   : if (autohash_exists($autohash,$key)) { ... }
  Function: Test whether key is present in $autohash.
  Args    : Hash::AutoHash object, key
  Returns : boolean
 
-=head3 autohash_each
-
  Title   : autohash_each
  Usage   : while (my($key,$value)=autohash_each($autohash)) { ... }
            -- OR --
@@ -1157,47 +1131,37 @@ functions in a 'use' statement.
  Returns : list context: next key=>value pair in $autohash or empty list at end
            scalar context: next key in $autohash or undef at end
 
-=head3 autohash_keys
-
  Title   : autohash_keys
  Usage   : @keys=autohash_keys($autohash)
  Function: Get all keys that are present in $autohash
  Args    : Hash::AutoHash object
  Returns : list of keys
 
-=head3 autohash_values
-
  Title   : autohash_values
  Usage   : @values=autohash_values($autohash)
  Function: Get the values of all keys that are present in $autohash
  Args    : Hash::AutoHash object
  Returns : list of values
 
-=head3 autohash_count
-
  Title   : autohash_count
  Usage   : $count=autohash_count($autohash)
  Function: Get the number keys that are present in $autohash
  Args    : Hash::AutoHash object
  Returns : number
 
-=head3 autohash_empty
-
  Title   : autohash_empty
  Usage   : if (autohash_empty($autohash)) { ... }
  Function: Test whether $autohash is empty
  Args    : Hash::AutoHash object
  Returns : boolean
 
-=head3 autohash_notempty
-
  Title   : autohash_notempty
  Usage   : if (autohash_notempty($autohash)) { ... }
  Function: Test whether $autohash is not empty. Complement of autohash_empty
  Args    : Hash::AutoHash object
  Returns : boolean
 
-=head2 Accessing the tied object: autohash_tied
+=head2 Accessing the tied object
 
 If a Hash::AutoHash object is tied, the application sometimes needs to
 access the object implementing the underlying tied hash.  The term
@@ -1401,7 +1365,7 @@ L<Hash::AsObject> alerted us to the danger of methods inherited from
 UNIVERSAL.
 
 L<Hash::AutoHash::Args>, L<Hash::AutoHash::MultiValued>,
-L<Hash::AutoHash::AVPairsSingle>, L<Hash::AutoHash::AVPairsMulti>,
+L<Hash::AutoHash::AVP_Single>, L<Hash::AutoHash::AVP_Multi>,
 L<Hash::AutoHash::Record> are subclasses each of which provides an
 object interface to a specific tied hash class.
 
@@ -1427,17 +1391,21 @@ progress on your bug as I make changes.
 
 =over 4
 
-=item * Overridden UNIVERSAL methods no longer supported
+=item * Overridden UNIVERSAL methods
 
-INCOMPATIBLE CHANGE: As of version 1.14, it is no longer
-possible to use method notation for keys with the same names
-as methods inherited from UNIVERSAL (the base class of
-everything). These are 'can', 'isa', 'DOES', and 'VERSION'.
-The reason is that as of Perl 5.9.3, calling UNIVERSAL
-methods as functions is deprecated and developers are
-encouraged to use method form instead. Previous versions of
-AutoHash are incompatible with CPAN modules that adopt this
-style.
+Hash::AutoHash overrides all methods inherited from UNIVERSAL (the
+base class of everything), namely 'can', 'isa', 'DOES', and
+'VERSION'. These are logically class methods, since they give the same
+result for every object in a given class, but the built-in
+implementation allows them to be invoked on objects. Our
+implementation treats class and object invocations differently: if the
+method is invoked on a class, we pass it to UNIVERSAL, but if it is
+invoked on an object, we interpret it as a request to access or change
+an element of the underlying hash.
+
+This will break code that expects the UNIVERSAL methods
+to work in the standard way.  If this proves to be a problem, we
+will provide a way to revert the offending behavior.
 
 =item * Tied hashes and serialization
 
